@@ -53,7 +53,7 @@ class GenericLocalLinkConnectionHook(base.ProcessingHook):
         except TypeError:
             LOG.warning("TLV value for TLV type %d not in correct"
                         "format, ensure TLV value is in "
-                        "hexidecimal format when sent to "
+                        "hexadecimal format when sent to "
                         "inspector", tlv_type, node_info=node_info)
             return
 
@@ -115,6 +115,7 @@ class GenericLocalLinkConnectionHook(base.ProcessingHook):
     def before_update(self, introspection_data, node_info, **kwargs):
         """Process LLDP data and patch Ironic port local link connection"""
         inventory = utils.get_inventory(introspection_data)
+        lldp_raw = introspection_data.get('lldp_raw') or {}
 
         ironic_ports = node_info.ports()
 
@@ -130,7 +131,7 @@ class GenericLocalLinkConnectionHook(base.ProcessingHook):
                           node_info=node_info, data=introspection_data)
                 continue
 
-            lldp_data = iface.get('lldp')
+            lldp_data = lldp_raw.get(iface['name']) or iface.get('lldp')
             if lldp_data is None:
                 LOG.warning("No LLDP Data found for interface %s",
                             mac_address, node_info=node_info,
@@ -161,5 +162,5 @@ class GenericLocalLinkConnectionHook(base.ProcessingHook):
                 node_info.patch_port(port, patches)
             except exceptions.BadRequestException as e:
                 LOG.warning("Failed to update port %(uuid)s: %(error)s",
-                            {'uuid': port.uuid, 'error': e},
+                            {'uuid': port.id, 'error': e},
                             node_info=node_info)
