@@ -14,7 +14,6 @@
 """Handling introspection data from the ramdisk."""
 
 import copy
-import datetime
 import os
 
 from oslo_config import cfg
@@ -56,7 +55,7 @@ def _store_logs(introspection_data, node_info):
         'uuid': node_info.uuid if node_info is not None else 'unknown',
         'mac': (utils.get_pxe_mac(introspection_data) or
                 'unknown').replace(':', ''),
-        'dt': datetime.datetime.utcnow(),
+        'dt': timeutils.utcnow(),
         'bmc': (utils.get_ipmi_address_from_data(introspection_data) or
                 'unknown')
     }
@@ -79,10 +78,25 @@ def _store_logs(introspection_data, node_info):
 
 def _find_node_info(introspection_data, failures):
     try:
-        return node_cache.find_node(
-            bmc_address=utils.get_ipmi_address_from_data(introspection_data),
-            mac=utils.get_valid_macs(introspection_data))
+        address = utils.get_ipmi_address_from_data(introspection_data)
+        v6address = utils.get_ipmi_v6address_from_data(introspection_data)
+        bmc_addresses = list(filter(None, [address, v6address]))
+        macs = utils.get_valid_macs(introspection_data)
+        return node_cache.find_node(bmc_address=bmc_addresses,
+                                    mac=macs)
     except utils.NotFoundInCacheError as exc:
+        if CONF.processing.permit_active_introspection:
+            try:
+                return node_cache.record_node(bmc_addresses=bmc_addresses,
+                                              macs=macs)
+            except utils.NotFoundInCacheError:
+                LOG.debug(
+                    'Active nodes introspection is enabled, but no node '
+                    'was found for MAC(s) %(mac)s and BMC address(es) '
+                    '%(addr)s; proceeding with discovery',
+                    {'mac': ', '.join(macs) if macs else None,
+                     'addr': ', '.join(filter(None, bmc_addresses)) or None})
+
         not_found_hook = plugins_base.node_not_found_hook_manager()
         if not_found_hook is None:
             failures.append(_('Look up error: %s') % exc)
@@ -140,7 +154,15 @@ def _filter_data_excluded_keys(data):
             if k not in _STORAGE_EXCLUDED_KEYS}
 
 
-def _store_data(node_uuid, data, processed=True):
+def store_introspection_data(node_uuid, data, processed=True):
+    """Store introspection data to the storage backend.
+
+    :param node_uuid: node UUID
+    :param data: Introspection data to be saved
+    :param processed: The type of introspection data, set to True means the
+                      introspection data is processed, otherwise unprocessed.
+    :raises: utils.Error
+    """
     introspection_data_manager = plugins_base.introspection_data_manager()
     store = CONF.processing.store_data
     ext = introspection_data_manager[store].obj
@@ -148,15 +170,23 @@ def _store_data(node_uuid, data, processed=True):
 
 
 def _store_unprocessed_data(node_uuid, data):
-    # runs in background
     try:
-        _store_data(node_uuid, data, processed=False)
+        store_introspection_data(node_uuid, data, processed=False)
     except Exception:
         LOG.exception('Encountered exception saving unprocessed '
                       'introspection data for node %s', node_uuid, data=data)
 
 
 def get_introspection_data(uuid, processed=True, get_json=False):
+    """Get introspection data from the storage backend.
+
+    :param uuid: node UUID
+    :param processed: Indicates the type of introspection data to be read,
+                      set True to request processed introspection data.
+    :param get_json: Specify whether return the introspection data in json
+                     format, string value is returned if False.
+    :raises: utils.Error
+    """
     introspection_data_manager = plugins_base.introspection_data_manager()
     store = CONF.processing.store_data
     ext = introspection_data_manager[store].obj
@@ -176,7 +206,6 @@ def process(introspection_data):
         # Locking is already done in find_node() but may be not done in a
         # node_not_found hook
         node_info.acquire_lock()
-
     if failures or node_info is None:
         msg = _('The following failures happened during running '
                 'pre-processing hooks:\n%s') % '\n'.join(failures)
@@ -184,7 +213,6 @@ def process(introspection_data):
             node_info.finished(istate.Events.error, error='\n'.join(failures))
         _store_logs(introspection_data, node_info)
         raise utils.Error(msg, node_info=node_info, data=introspection_data)
-
     LOG.info('Matching node is %s', node_info.uuid,
              node_info=node_info, data=introspection_data)
 
@@ -193,12 +221,9 @@ def process(introspection_data):
         raise utils.Error(_('Node processing already finished with '
                             'error: %s') % node_info.error,
                           node_info=node_info, code=400)
-
-    # Note(mkovacik): store data now when we're sure that a background
-    # thread won't race with other process() or introspect.abort()
-    # call
-    utils.executor().submit(_store_unprocessed_data, node_info.uuid,
-                            unprocessed_data)
+    # NOTE(TheJulia): this was previously called as a background
+    # process, but we can't do that with sqlite.
+    _store_unprocessed_data(node_info.uuid, unprocessed_data)
 
     try:
         node = node_info.node()
@@ -240,9 +265,10 @@ def _run_post_hooks(node_info, introspection_data):
 @node_cache.fsm_transition(istate.Events.process, reentrant=False)
 def _process_node(node_info, node, introspection_data):
     # NOTE(dtantsur): repeat the check in case something changed
-    ir_utils.check_provision_state(node)
+    keep_power_on = ir_utils.check_provision_state(node)
+
     _run_post_hooks(node_info, introspection_data)
-    _store_data(node_info.uuid, introspection_data)
+    store_introspection_data(node_info.uuid, introspection_data)
 
     ironic = ir_utils.get_client()
     pxe_filter.driver().sync(ironic)
@@ -250,10 +276,15 @@ def _process_node(node_info, node, introspection_data):
     node_info.invalidate_cache()
     rules.apply(node_info, introspection_data)
 
-    resp = {'uuid': node.uuid}
+    resp = {'uuid': node.id}
 
+    # determine how to handle power
+    if keep_power_on or not node_info.manage_boot:
+        power_action = False
+    else:
+        power_action = CONF.processing.power_off
     utils.executor().submit(_finish, node_info, ironic, introspection_data,
-                            power_off=CONF.processing.power_off)
+                            power_off=power_action)
 
     return resp
 
@@ -263,7 +294,7 @@ def _finish(node_info, ironic, introspection_data, power_off=True):
     if power_off:
         LOG.debug('Forcing power off of node %s', node_info.uuid)
         try:
-            ironic.node.set_power_state(node_info.uuid, 'off')
+            ironic.set_node_power_state(node_info.uuid, 'power off')
         except Exception as exc:
             if node_info.node().provision_state == 'enroll':
                 LOG.info("Failed to power off the node in"
@@ -292,13 +323,13 @@ def reapply(node_uuid, data=None):
     stored data.
 
     :param node_uuid: node UUID
+    :param data: unprocessed introspection data to be reapplied
     :raises: utils.Error
-
     """
 
     LOG.debug('Processing re-apply introspection request for node '
               'UUID: %s', node_uuid)
-    node_info = node_cache.get_node(node_uuid, locked=False)
+    node_info = node_cache.get_node(node_uuid)
     if not node_info.acquire_lock(blocking=False):
         # Note (mkovacik): it should be sufficient to check data
         # presence & locking. If either introspection didn't start
@@ -307,7 +338,7 @@ def reapply(node_uuid, data=None):
         raise utils.Error(_('Node locked, please, try again later'),
                           node_info=node_info, code=409)
 
-    utils.executor().submit(_reapply, node_info, data)
+    utils.executor().submit(_reapply, node_info, introspection_data=data)
 
 
 def _reapply(node_info, introspection_data=None):
@@ -350,6 +381,6 @@ def _reapply_with_data(node_info, introspection_data):
                           '\n'.join(failures), node_info=node_info)
 
     _run_post_hooks(node_info, introspection_data)
-    _store_data(node_info.uuid, introspection_data)
+    store_introspection_data(node_info.uuid, introspection_data)
     node_info.invalidate_cache()
     rules.apply(node_info, introspection_data)
