@@ -65,6 +65,20 @@ class BaseAPITest(test_base.BaseTest):
 
 class TestApiIntrospect(BaseAPITest):
 
+    def test_root_endpoint_with_prefix(self):
+        res = self.app.get(
+            '/', headers={'X-Forwarded-Proto': 'https',
+                          'X-Forwarded-Host': 'myhost',
+                          'X-Forwarded-Prefix': '/ironic_inspector'})
+
+        self.assertEqual(200, res.status_code)
+        self.assertEqual('application/json',
+                         res.headers['content-type'])
+        cur_version = res.json['versions'][0]
+        hrefs = [link['href'] for link in cur_version['links']]
+        for href in hrefs:
+            self.assertTrue(href.startswith('https://myhost/ironic_inspector'))
+
     def test_introspect(self):
 
         res = self.app.post('/v1/introspection/%s' % self.uuid,
@@ -329,19 +343,50 @@ class TestApiListStatus(GetStatusAPIBaseTest):
         self.assertEqual([self.finished_node.status,
                           self.unfinished_node.status], statuses)
         list_mock.assert_called_once_with(marker=None,
-                                          limit=CONF.api_max_limit)
+                                          limit=CONF.api_max_limit, state=None)
 
     def test_list_introspection_limit(self, list_mock):
         res = self.app.get('/v1/introspection?limit=1000')
         self.assertEqual(200, res.status_code)
-        list_mock.assert_called_once_with(marker=None, limit=1000)
+        list_mock.assert_called_once_with(marker=None, limit=1000, state=None)
 
     def test_list_introspection_makrer(self, list_mock):
         res = self.app.get('/v1/introspection?marker=%s' %
                            self.finished_node.uuid)
         self.assertEqual(200, res.status_code)
         list_mock.assert_called_once_with(marker=self.finished_node.uuid,
-                                          limit=CONF.api_max_limit)
+                                          limit=CONF.api_max_limit, state=None)
+
+    def test_list_introspection_state(self, list_mock):
+        state = 'error'
+        list_mock.return_value = [self.finished_node]
+        res = self.app.get('/v1/introspection?state=%s' % state)
+        self.assertEqual(200, res.status_code)
+        statuses = json.loads(res.data.decode('utf-8')).get('introspection')
+
+        self.assertEqual([self.finished_node.status], statuses)
+        list_mock.assert_called_once_with(marker=None,
+                                          limit=CONF.api_max_limit,
+                                          state=state.split(','))
+
+    def test_list_introspection_multiple_state(self, list_mock):
+        state = 'error,processing'
+        list_mock.return_value = [self.finished_node,
+                                  self.unfinished_node]
+        res = self.app.get('/v1/introspection?state=%s' % state)
+        self.assertEqual(200, res.status_code)
+        statuses = json.loads(res.data.decode('utf-8')).get('introspection')
+
+        self.assertEqual([self.finished_node.status,
+                          self.unfinished_node.status], statuses)
+        list_mock.assert_called_once_with(marker=None,
+                                          limit=CONF.api_max_limit,
+                                          state=state.split(','))
+
+    def test_list_introspection_invalid_state(self, list_mock):
+        state = 'error,invalid'
+        res = self.app.get('/v1/introspection?state=%s' % state)
+        self.assertEqual(400, res.status_code)
 
 
 class TestApiGetData(BaseAPITest):
@@ -597,6 +642,12 @@ class TestApiRules(BaseAPITest):
         delete_mock.assert_called_once_with(self.uuid)
         self.assertEqual('text/plain; charset=utf-8',
                          res.headers['content-type'])
+        # NOTE(TheJulia): This should be able to be removed once
+        # https://github.com/eventlet/eventlet/issues/746
+        # is resolved and available. This ensures we don't
+        # send out a content length header out which is the
+        # less evil of the RFC7230 broken-ness.
+        self.assertEqual('0', res.headers['Content-Length'])
 
 
 class TestApiMisc(BaseAPITest):
@@ -712,6 +763,11 @@ class TestApiVersions(BaseAPITest):
                    main._format_version(main.CURRENT_API_VERSION)}
         self._check_version_present(self.app.get('/', headers=headers))
 
+    def test_request_correct_version_with_standard_singular_header(self):
+        headers = {'OpenStack-API-Version': 'baremetal-introspection %s' %
+                   main._format_version(main.CURRENT_API_VERSION)}
+        self._check_version_present(self.app.get('/', headers=headers))
+
     def test_request_unsupported_version(self):
         bad_version = (main.CURRENT_API_VERSION[0],
                        main.CURRENT_API_VERSION[1] + 1)
@@ -731,6 +787,12 @@ class TestApiVersions(BaseAPITest):
         self.assertEqual(200, res.status_code)
         self._check_version_present(res)
 
+    def test_request_latest_version_with_standard_singular_header(self):
+        headers = {'OpenStack-API-Version': 'baremetal-introspection latest'}
+        res = self.app.get('/', headers=headers)
+        self.assertEqual(200, res.status_code)
+        self._check_version_present(res)
+
 
 class TestPlugins(unittest.TestCase):
     @mock.patch.object(example_plugin.ExampleProcessingHook,
